////////////////////////////////////////////////////////
//
//    Radar.java
// Illustration of threads 
// All of the following are threads in these game:
// Machine Gun, Balloon Releaser, Balloons and Bullets
// Machine Gun does not fire unless balloons are between the red lines (or
// triggered by manually from the "Manual Fire" button)
//
////////////////////////////////////////////////////////
import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;

public class Radar extends Applet implements ActionListener
{
private Button  ceaseFRButton, commandFRButton,fireButton, stopButton;

private BalloonMachine balloons;
private AutomaticGun machineGun;
private int lowerRange,upperRange;
private int inzone=0;
private int bullet=0;
public Point redPoint;
public Rectangle redZone;
public Rectangle scan;
  
  
  // method init
public void init()
  {
    setFont(new Font("SansSerif",Font.BOLD,12));
    ceaseFRButton = new Button("CEASE FIRING");
    ceaseFRButton.addActionListener(this);
    ceaseFRButton.setBackground(Color.yellow); 
    add(ceaseFRButton);

    commandFRButton = new Button("COMMAND FIRING");
    commandFRButton.addActionListener(this);
    commandFRButton.setBackground(Color.yellow); 
    add(commandFRButton);

    fireButton = new Button("MANUAL FIRE");
    fireButton.addActionListener(this);
    fireButton.setBackground(Color.yellow); 
    add(fireButton);

    lowerRange=getSize().height/2-50;
    upperRange=lowerRange+100;
    setBackground(Color.lightGray);
    inzone=0; bullet=0;
    balloons = new BalloonMachine(this);
    machineGun= new AutomaticGun(this);
    
    redPoint=new Point(30,getSize().height/2);
    redZone=new Rectangle(15,getSize().height/2-5,30,10);
    scan=new Rectangle(0,lowerRange,100,100);


    inzone=0;bullet=0;
    balloons.start();
    machineGun.start();

  }

  // stop method
public void stop()
  {
    balloons.stop();
    machineGun.stop();
  }

  // modifies actionPerformed to control the buttons
public void actionPerformed(ActionEvent e)
  {
    if (e.getActionCommand().equals("CEASE FIRING"))
      {
	if (machineGun.isAlive())
	  machineGun.suspend();
      }
    else if (e.getActionCommand().equals("COMMAND FIRING"))
      {
	if (machineGun.isAlive())
	  machineGun.resume();
      }
    else if (e.getActionCommand().equals("MANUAL FIRE"))
      {
	machineGun.fire();
      }
  }
  // paint method
public void paint(Graphics g)
  {

    int w=getSize().width;
    int h=getSize().height;
    g.setColor(Color.red);
    g.fillOval(w-5,lowerRange-2,5,5);
    g.fillOval(w-5,upperRange-2,5,5);
    g.drawLine(0,lowerRange,w,lowerRange);
    g.drawLine(0,upperRange,w,upperRange);
    g.setColor(Color.black);
    g.fillRect(w-50,h/2-5,50,5);
    g.fillRect(w-30,h/2,50,3);
    g.setColor(Color.yellow);
    g.fillRect(5,h-30,10,30);
    g.setColor(Color.green);
    g.fillRect(45,h-30,10,30);
    g.setColor(Color.darkGray);
    g.fillRect(0,h-5,w,5);

  }

  // machine gun will not fire if there is no balloon between the red lines
  //
public synchronized boolean detected(boolean object) 
  {
    if (object)
      inzone++;
    else {
      while (inzone <1) //no balloons
	{  //no balloons
	  try
	    {
	      wait();
	    }
	  catch(InterruptedException e)
	    {
	      System.err.println(e.toString());
	    }
	}//while
    } //else
    notify();
    return true;
  }
public synchronized void leftScan()
  {
    inzone--;
  }
  
public void atTarget()
  {
    bullet++; 
  }
public void leftTarget()
  {
    bullet--;
  }

public int checkRedZone()
  {
    return bullet;
  }
     
public boolean getTarget()
  {
    return bullet > 0;
  }
}//end class



//**************************************************
//**************************************************
//**************************************************

class AutomaticGun extends Thread 
{
private Radar platform;
private   Image image;
  
public AutomaticGun(Radar  p){
    platform=p;
    prepareBulletImage();
}
  // run method
public void run() {
  while (true)
    {
      try
        {
          Thread.sleep(500) ;
	}
        catch(InterruptedException e)
	  {
	    System.err.println(e.toString());
	  }
        commandFiring();
    }
}
public void commandFiring()
  {
    if (platform.detected(false)) 
      fire();
  }
public void fire()
  {
    Bullet b= new Bullet(platform,image);
    b.start();
  }
public void prepareBulletImage()
  {
    // creates image to used in double buffering
    image = platform.createImage(15,5);
    Graphics og = image.getGraphics();
    og.setColor(Color.lightGray);
    og.fillOval(10,10,5,5);
    og.setColor(Color.red);
    og.fillOval(0,0,5,5);
    og.dispose();
  }

}
//**************************************************
//   <<<<++++++++ BUllet
//**************************************************

class Bullet extends Thread 
{
private Radar platform;
private int x,y;
private int dx=-10;
private boolean enteredRedZone ,inRedZone;
private   Image image;
  
public Bullet(Radar p,Image i)
  {
    platform=p;
    y=platform.getSize().height/2-3;
    x=platform.getSize().width-53;
    enteredRedZone=inRedZone=false;
    image=i;
}
  // run method
public void run() {
  while (true)
    {
      try
        {
          Thread.sleep(15) ;
	}
      catch(InterruptedException e)
	{
	  System.err.println(e.toString());
	  }
      move();
    }
}
public void move()
  {
    x+=dx;
    if (x <-15)
      die();
    paint();
    if (!enteredRedZone) {
      if (inRedZone=enteredRedZone=checkRedZone())
	platform.atTarget();
    }
    else if (inRedZone){
      inRedZone=checkRedZone();
      if (!inRedZone)
	platform.leftTarget();
    }
  }

public void paint()
  {
    Graphics g= platform.getGraphics();
    g.drawImage(image, x, y,platform);
  }
public void die()
  {
    stop();
  }
public boolean checkRedZone()
  {
    if ( platform.redZone.contains(x,y) )
      return true;
    return false;
  }
} //end Bullet
//**************************************************
//**************************************************
//**************************************************
//**************************************************
class BalloonMachine extends Thread 
{
  private Radar platform;
  private int balloons;
  private   Image image;
  
  
  public  BalloonMachine(Radar p)
  {
    platform=p;
    prepareChildImage();
    
  }
  // run method
  public void run()
  {
    balloons=0;
    while (true)
    {
        try
        {
          Thread.sleep( (int)(  Math.random()*1000 ) +1000) ;
        }
        catch(InterruptedException e)
        {
          System.err.println(e.toString());
        }
        makeBalloon();
      }
  }
public void makeBalloon()
  {
    if (balloons >3)   // hasTooMuchBalloon())
      return;
    Balloon b= new Balloon(platform,this);
    b.start();
    balloons++;
  }
public void myChildIsDying()
  {
    balloons--; //:-)
  }
public void prepareChildImage()
  {
    // creates image to used in double buffering
    image = platform.createImage(33,33);
    Graphics og = image.getGraphics();
    og.setColor(Color.lightGray);
    og.fillOval(3,3,30,30);
    og.setColor(Color.blue);
    og.fillOval(0,0,30,30);
    og.dispose();
  }
public Image myChildFace()
  {
    return image;
  }
  
 
}

//**************************************************

class Balloon extends Thread 
{
private Radar platform;
private int x,y;
private int dy=-3;
private int sleeptime;
private  BalloonMachine father ;
private boolean scanned,inScan;
private Image image;
  
  
public Balloon(Radar p,BalloonMachine f)
  {
    platform=p;
    x=15;
    y=platform.getSize().height;
    sleeptime= (int) (Math.random() * 30) +30;
    father=f;
    scanned=inScan=false;
    image=f.myChildFace();
}
  // run method
public void run() {
  while (true)
    {
      try
        {
          Thread.sleep(sleeptime) ;
	}
      catch(InterruptedException e)
	{
	  System.err.println(e.toString());
	  }
      move();
    }
}
public void move()
  {
    y+=dy;
    if (y <-33)
      die();
    paint();
    checkRedZone();
    if (!scanned)
      inScan=scanned=checkRadarScan();
    else if (inScan)
      inScan=checkRadarScan();
  }
public void paint()
  {
    Graphics g= platform.getGraphics();
    g.drawImage(image, x, y,platform);
  }
public void die()
  {
    father.myChildIsDying();
    stop();
  }
public void erase(){
      Graphics g = platform.getGraphics();
      g.setColor(Color.lightGray);
      g.fillOval(x,y,30,30);
}
  
public void checkRedZone()
  {
    Rectangle r= new Rectangle(x,y,30,30);
    if ( platform.getTarget() && r.contains(platform.redPoint) ) {
      erase();
      platform.leftScan();
      die();
    }
  }
public boolean     checkRadarScan(){
  if (platform.scan.contains(x+15,y+15)) {
    if (!scanned)
      platform.detected(true);
    return true;
  }
  if (inScan)
    platform.leftScan();
  return false;
  
}
  
}//class


